'use strict';

var babelCore = require('babel-core');
var RawModule = require('webpack/lib/RawModule');
var SingleEntryDependencyWrapper = require('../SingleEntryDependencyWrapper');
var _Symbol = require('symbol');
var NullFactory = require('webpack/lib/NullFactory');

var moduleIdentifierSymbol = _Symbol('topLevelModuleIdentifier');

module.exports = function (compiler, pluginOptions) {
    compiler.hooks.compilation.tap('webpackBabelExternalHelpersPlugin', function (compilation) {
        inject(compilation, pluginOptions);
    });
};

function inject(compilation, pluginOptions) {
    compilation.dependencyFactories.set(SingleEntryDependencyWrapper, new NullFactory());
    compilation.hooks.seal.tap('webpackBabelExternalHelpersPlugin', function () {
        getChunks(compilation, pluginOptions.entries).forEach(function (chunk) {
            return injectHelpers(compilation, chunk.module, pluginOptions.whitelist);
        });
    });
}

function getChunks(compilation, onlyEntries) {
    var chunks = compilation._preparedEntrypoints;
    if (onlyEntries.length < 1) {
        return chunks;
    }

    return chunks.filter(function (chunk) {
        return onlyEntries.indexOf(chunk.name) > -1;
    });
}

function injectHelpers(compilation, topLevelModule, babelHelpersWhitelist) {
    var dependency = getDependency(topLevelModule);

    var _compilation$addModul = compilation.addModule(createHelpersModule(dependency, babelHelpersWhitelist)),
        module = _compilation$addModul.module,
        shouldBuild = _compilation$addModul.build;

    link(topLevelModule, module, dependency);
    if (shouldBuild) {
        topLevelModule.dependencies.unshift(dependency);
        compilation.buildModule(module, false, topLevelModule, [dependency], function (error) {
            if (error) {
                throw new Error('Helpers module is not build.');
            }
        });
    }
}

function getDependency(module) {
    return findInjectedDependency(module) || createDependency(module);
}

function findInjectedDependency(module) {
    return module.dependencies.filter(function (dependency) {
        return dependency instanceof SingleEntryDependencyWrapper;
    })[0];
}

function createDependency(module) {
    var dependency = new SingleEntryDependencyWrapper('babel-loader-external-helpers');
    dependency.loc = module.name + ":" + (100000 - 1);
    dependency[moduleIdentifierSymbol] = module.identifier();

    return dependency;
}

function createHelpersModule(dependency, whitelist) {
    return new RawModule(buildBabelHelpers(whitelist), getModuleIdentifier(dependency));
}

function buildBabelHelpers(whitelist) {
    if (whitelist.length) {
        return babelCore.buildExternalHelpers(whitelist);
    }

    return babelCore.buildExternalHelpers();
}

function getModuleIdentifier(dependency) {
    return 'generated babel-helpers ' + dependency[moduleIdentifierSymbol];
}

function link(topLevelModule, module, dependecy) {
    module.issuer = topLevelModule;
    dependecy.module = module;
    module.addReason(topLevelModule, dependecy);
}